#Autogenerated by ReportLab guiedit do not edit
from reportlab.graphics.charts.piecharts import Pie
from reportlab.graphics.shapes import Drawing, _DrawingEditorMixin
from reportlab.lib.colors import black

class Drawing_000(_DrawingEditorMixin,Drawing):
	def __init__(self,width=400,height=200,*args,**kw):
		Drawing.__init__(self,width,height,*args,**kw)
		self._add(self,Pie(),name=None,validate=None,desc=None)
		self.contents[0].slices[0].strokeColor        = black
		self.contents[0].slices[0].strokeWidth        = 1

if __name__=="__main__": #NORUNTESTS
	Drawing_000().save(formats=['pdf'],outDir='.',fnRoot=None)
